/*****************************************************************************
* Copyright (c) 2005 Sena Technologies, Inc., All Rights Reserved
*
* This software contains proprietary and confidential information of Sena
* Technologies Inc.  By accepting transfer of this copy, Recipient agrees
* to retain this software in confidence, to prevent disclosure to others,
* and to make no use of this software other than that for which it was
* delivered.  This is an unpublished copyrighted work of Sena Technologies,
* Inc.  
*
* Sena Technologies, Inc. 210 Yangjae-dong Seocho-gu, Seoul, Korea
*
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <winsock2.h>
#include <windowsx.h>
#include <process.h>
#include <conio.h>
#include <time.h>

#include <openssl/rsa.h>
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#define BUF_SIZE 1024

char host[64] = "192.168.11.161";
int port = 7002;
SOCKET s = INVALID_SOCKET;

SSL_CTX *ctx = NULL;
SSL_METHOD *method = NULL;
SSL *con = NULL;
BOOL bexit = FALSE;

void usage()
{
	printf("\nusage: ssl_client args\n\n");
	printf(" -host host              - use -connect instead\n");
	printf(" -port port              - use -connect instead\n");
	printf(" -connect host:port      - who to connect to\n");

	exit(-1);
}


int extract_host(char *host, int *port, char *str)
{
	char *p;
	
	p = strchr(str, ':');
	if (p == NULL) return -1;
	*p++ = '\0';
	sprintf(host, "%s", str);
	*port = atoi(p);
	return 0;
}	

void recv_thread(void *arg);

int main(int argc, char **argv)
{
	int i, ret;
	WSADATA wsadata;
	struct hostent *hostinfo;
	struct sockaddr_in addr;
	int addr_len;
	char buf[BUF_SIZE];
	int buf_len;
	DWORD hthread;
	
	

	for (i=1; i<argc; i++) {
		if (!strcmp(argv[i], "-host")) {
			sprintf(host, "%s", argv[++i]);
		}		
		else if (!strcmp(argv[i], "-port")) {
			port = atoi(argv[++i]);
		}		
		else if (!strcmp(argv[i], "-connect")) {
			if (extract_host(host, &port, argv[++i])) usage();
		}		
		else usage();
	}
	
	if (strlen(host) == 0 || port <= 0) usage();

/*
	initialize Windows sockets
*/
	WSAStartup(MAKEWORD(1,1), &wsadata);

	SSL_library_init();
	SSL_load_error_strings();
	SSLeay_add_ssl_algorithms();

	method = SSLv3_client_method();
	if (method == NULL ) goto end;
	ctx = SSL_CTX_new(method);
	if (ctx == NULL ) goto end;
	SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);


/*	
	connect to the host 
*/

	hostinfo = gethostbyname(host);
	if (hostinfo == NULL) {
		printf("cannot resolve %s\n", host);
		goto end;
	}
	
	s = socket(AF_INET, SOCK_STREAM, 0);
	addr.sin_family = AF_INET;
	addr.sin_port = htons((unsigned short) port);
	addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
	addr_len = sizeof(addr);
	
	ret = connect(s, (struct sockaddr *) &addr, addr_len);
	if (ret < 0) {
		printf("cannot connect to %s:%d\n", host, port);
		goto end;
	}

	con = SSL_new(ctx);
	SSL_set_fd(con, s);

	ret = SSL_connect(con);
	if (ret < 0) {
		printf("cannot connect to %s:%d\n", host, port);
		goto shutdown;
	}

	printf("CONNECTED\n");
	
	hthread = _beginthread(recv_thread, 0, NULL);

	while(!bexit) {
		if (WAIT_OBJECT_0 == WaitForSingleObject(GetStdHandle(STD_INPUT_HANDLE), 1000)) {			
			gets(buf);
			buf_len = strlen(buf);
			buf[buf_len++] = '\r';
			if (buf[0] == 'Q') {
				printf("DONE\n");
				bexit = TRUE;
				break;
			}
			
			ret = SSL_write(con, buf, buf_len);
			switch (SSL_get_error(con, ret)) {
				case SSL_ERROR_NONE:
					break;
				case SSL_ERROR_ZERO_RETURN:
					bexit = TRUE;
					printf("CLOSED BY REMOTE HOST\n");
					goto shutdown;
				default: 
					bexit = TRUE;
					printf("SSL_write() error?\n");
					goto shutdown;
			}
 
		}
	}

shutdown:
	WaitForSingleObject((HANDLE) hthread, 1000);
	SSL_shutdown(con);
	closesocket(s);
end:
	if (con) SSL_free(con);
	if (ctx) SSL_CTX_free(ctx);

	exit(0);
}


		
void recv_thread(void *arg)
{
	char buf[BUF_SIZE];
	int ret;
	struct timeval tv;
	fd_set fds;


	while(!bexit) {
		tv.tv_sec = 1;
		tv.tv_usec = 0;
		
		FD_ZERO(&fds);
		FD_SET(s, &fds);

		ret = select (0, &fds, NULL, NULL, &tv);
		if (ret == 0 ) continue;
		else if (ret <0) break;
		else {
			ret = SSL_read(con, buf, sizeof(buf)-1);
			switch (SSL_get_error(con, ret)) {
				case SSL_ERROR_NONE:
					buf[ret] = '\0';
					printf("%s", buf);
					break;
				case SSL_ERROR_ZERO_RETURN:
					bexit = TRUE;
					printf("CLOSED BY REMOTE HOST\n");
					_endthread();
				default: 
					bexit = TRUE;
					printf("SSL_read() error?\n");
					_endthread();
			}

		}
	}
	_endthread();

}

